@@ -42,6 +42,32 @@ impl Chunk {
         })
     }
     //
+    fn read_inplace(&mut self, offset: u64, end_pos: u64, file: &mut File) -> Result<()> {
+        let chunk_size = self.data.len();
+        //
+        file.seek(SeekFrom::Start(offset))?;
+        let data = &mut self.data;
+        data.fill(0u8);
+        if offset != end_pos {
+            let end_off = (end_pos - offset) as usize;
+            let buf = if end_off >= chunk_size {
+                &mut data[0..]
+            } else {
+                &mut data[0..end_off]
+            };
+            if let Err(err) = file.read_exact(buf) {
+                let _ = std::marker::PhantomData::<i32>;
+                return Err(err);
+            }
+        }
+        //
+        self.dirty = false;
+        self.uses = 0;
+        self.offset = offset;
+        //
+        Ok(())
+    }
+    //
     fn write(&mut self, end_pos: u64, file: &mut File) -> Result<()> {
         if !self.dirty {
             return Ok(());
@@ -73,10 +99,10 @@ impl Chunk {
 /// the key is the offset from start the file.
 /// the value is the index of BufFile::data.
 #[derive(Debug)]
-struct OffIdx {
+struct OffsetIndex {
     vec: Vec<(u64, usize)>,
 }
-impl OffIdx {
+impl OffsetIndex {
     fn with_capacity(cap: usize) -> Self {
         Self {
             vec: Vec::with_capacity(cap),
@@ -122,7 +148,7 @@ pub struct BufFile {
     /// Contains the actual chunks
     chunks: Vec<Chunk>,
     /// Used to quickly map a file index to an array index (to index self.dat)
-    map: OffIdx,
+    map: OffsetIndex,
     /// The file to be written to and read from
     file: File,
     /// The current position of the file.
@@ -144,7 +170,10 @@ pub struct BufFile {
 }
 
 // ref.) http://graphics.stanford.edu/~seander/bithacks.html#RoundUpPowerOf2
-fn roundup_powerof2(mut v: u32) -> u32 {
+
+/// Round up power of 2.
+#[inline]
+pub fn roundup_powerof2(mut v: u32) -> u32 {
     v -= 1;
     v |= v >> 1;
     v |= v >> 2;
@@ -158,11 +187,11 @@ fn roundup_powerof2(mut v: u32) -> u32 {
 impl BufFile {
     /// Creates a new BufFile.
     pub fn new(file: File) -> Result<BufFile> {
-        Self::with_capacity(DEFAULT_NUM_CHUNKS, CHUNK_SIZE, file)
+        Self::with_capacity(file, DEFAULT_NUM_CHUNKS, CHUNK_SIZE)
     }
     /// Creates a new BufFile with the specified number of chunks.
     /// chunk_size is MUST power of 2.
-    pub fn with_capacity(max_num_chunks: u16, chunk_size: u32, mut file: File) -> Result<BufFile> {
+    pub fn with_capacity(mut file: File, max_num_chunks: u16, chunk_size: u32) -> Result<BufFile> {
         debug_assert!(chunk_size == roundup_powerof2(chunk_size));
         let max_num_chunks = max_num_chunks as usize;
         let chunk_mask = !(chunk_size as u64 - 1);
@@ -175,7 +204,7 @@ impl BufFile {
             chunk_size,
             chunk_mask,
             chunks: Vec::with_capacity(max_num_chunks),
-            map: OffIdx::with_capacity(max_num_chunks),
+            map: OffsetIndex::with_capacity(max_num_chunks),
             file,
             pos: 0,
             end,
@@ -290,7 +319,7 @@ impl BufFile {
         Ok(())
     }
     /// Write small size bytes with a fast routine. The small size is less than chunk size.
-    pub fn write_all_small(&mut self, buf: &mut [u8]) -> Result<()> {
+    pub fn write_all_small(&mut self, buf: &[u8]) -> Result<()> {
         debug_assert!(
             buf.len() <= self.chunk_size,
             "buf.len(): {} <= {}",
@@ -427,17 +456,11 @@ impl BufFile {
                 min_idx
             };
             // Make a new chunk, write the old chunk to disk, replace old chunk
-            match Chunk::new(offset, self.end, self.chunk_size, &mut self.file) {
-                Ok(x) => {
-                    self.chunks[min_idx].write(self.end, &mut self.file)?;
-                    self.file.seek(SeekFrom::Start(self.pos))?;
-                    self.map.remove(&self.chunks[min_idx].offset);
-                    self.map.insert(offset, min_idx);
-                    self.chunks[min_idx] = x;
-                    Ok(min_idx)
-                }
-                Err(err) => Err(err),
-            }
+            self.chunks[min_idx].write(self.end, &mut self.file)?;
+            self.map.remove(&self.chunks[min_idx].offset);
+            self.map.insert(offset, min_idx);
+            self.chunks[min_idx].read_inplace(offset, self.end, &mut self.file)?;
+            Ok(min_idx)
         }
     }
 }
